from passlib.hash import md5_crypt
from network_hash_gen.base import BaseHash


class Type5(BaseHash):
    """
    Calculates the md5-crypt based type 5 hashes for Cisco IOS/IOS-XE.
    """

    salt_chars = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    salt_length = 4

    @classmethod
    def hash_salted(cls, password: str, salt: str) -> str:
        """
        Calculates a Cisco IOS/IOS-XE Type 5 hash with the given password and
        salt.

        This function assumes that the given salt is valid for that hash.
        Using an invalid salt leads to undefined behaviour.
        Prefer the `hash` and `hash_seeded` functions over this
        one if possible.
        """

        m = md5_crypt.using(salt=salt).hash(password)
        return m



from unittest import TestCase

from network_hash_gen.utils import _generate_salt


class TestGenerateSalt(TestCase):
    def test_single_char_salt(self):
        self.assertEqual("aaaa", _generate_salt("a", 4))
        self.assertEqual("aaaaaaaa", _generate_salt("a", 8))
        self.assertEqual("aaaa", _generate_salt("a", 4, "someseed"))

    def test_reproducible_salts(self):
        """
        This contains several precalculated salts to ensure that this function
        returns the same output in the future.
        """
        self.assertEqual("eefgfhgf", _generate_salt("abcdefgh", 8, seed="testseed"))
        self.assertEqual("ebgghhbh", _generate_salt("abcdefgh", 8, seed="testseed2"))
        self.assertEqual("ehcahbfh", _generate_salt("abcdefgh", 8, seed="testseed3"))
        self.assertEqual("ecbbfbbd", _generate_salt("abcdefgh", 8, seed="testseed4"))



from network_hash_gen.cisco_ios import Type5
from tests.hash_test_suite import BaseHashTest


class TestCisocIOSType5(BaseHashTest.HashTest):
    def setUp(self):
        self.group = Type5
        self.hash_password_pairs = [
            # ASR920, IOS-XE 16.09.05f
            ("$1$u5Qz$ASoUsdGV2sOmi8XYkTuex1", "0foo0"),
            ("$1$Awu/$1X3gez8VPPh9.McBXfA1..", "1foo1"),
            ("$1$7R0z$CUJUEZXE/CS0kHkyGWtcb/", "2foo2"),
            ("$1$WF3X$GaRbNTz64GhwYHG0PquQw/", "3foo3"),
            ("$1$8OAY$GAKB/Yfz3ua4JCHfUAoUo0", "4foo4"),
            ("$1$Z2yX$hTf/MOQQYpaRCCFMJ1Wo//", "5foo5"),
            ("$1$OA7L$25A2lP5L0xvdIUu.8asgx/", "6foo6"),
            ("$1$3.ND$cRS/.rSfCIlwGuW0H3YzW.", "7foo7"),
            ("$1$tVk7$JAnQZwfmGZBk4TslhclFY.", "8foo8"),
            ("$1$262O$Hyxgu5up0M6a7nbsUZXmu/", "9foo9"),
            # C9300, IOS-XE 16.09.05
            ("$1$FCLB$2a3ny5w.KvoBl9WtjJePx1", "0foo0"),
            ("$1$bR4d$TcOVI.MROLfU2Gh4aXQrb0", "1foo1"),
            ("$1$Hl7d$LvCxzKmSxvv0ueXHTF6FO/", "2foo2"),
            ("$1$nXcV$JZEq6Mq639ef1eAKWiOxS1", "3foo3"),
            ("$1$C1ji$9Tfj7L4m5gDr5U2mvjfWU1", "4foo4"),
            ("$1$rRaQ$GvbTY9nLhVDm7CjvS/1yM.", "5foo5"),
            ("$1$vkQG$623sdz.eF3/6ueT.yiwRl0", "6foo6"),
            ("$1$ebEh$Me78poVDb0tfYX//MuYsU/", "7foo7"),
            ("$1$ezpt$YNP.lTuCJFI/CSYtQuicz.", "8foo8"),
            ("$1$t0yF$gNSfdCyGLt5kWicAv7qgf0", "9foo9"),
        ]

        self.hash_password_seed_tuples = [
            ("$1$v4mc$2Suu.8oPyYRG0.XUfV75D0", "asdfasdfasdf", "tzuijhuikj"),
            ("$1$UMmK$dKZmb7om9b6O/E/qlRILM0", "123sd()123!", "78/(123ksdiASDij"),
        ]



from tests.hash_test_suite import BaseHashTest
from network_hash_gen.cisco_ios import Type9


class TestCiscoIOSType9(BaseHashTest.HashTest):
    def setUp(self):
        self.group = Type9
        self.hash_password_pairs = [
            # ASR920, IOS-XE 16.09.05f
            (
                "$9$plz/kf0e3.dns.$cKvJDd6bXmyaUWdx42dgcRwpdDs0djBEqzjlmMbYEW6",
                "0foo0",
            ),
            (
                "$9$8Yn1IbhUG.YOGk$B0Tce.G3lMqSLb.qhSwQwxyi4qBU3jSlH2qTIelK.A2",
                "1foo1",
            ),
            (
                "$9$PMqVmgmR4Nvlsk$JzTbAxkZuDpMGImpXHGysj6uWjcQnikXR0GBdUhmdxw",
                "2foo2",
            ),
            (
                "$9$iMKNK2MyDfqPsU$V443Y/yutR./oArWmQA7pj6BNp5TrgC0MmGZ7VIKff2",
                "3foo3",
            ),
            (
                "$9$MigKM5HW.PsFlU$nUUsOfUnGck.G6q.9j0gqlb4vXDQIihOlkYaFuQ0/0g",
                "4foo4",
            ),
            (
                "$9$5vByDSrtKlVj8.$RhjgPmH7GTJpjbb3brYqQOXtP.UIpMiqS644VA.N12A",
                "5foo5",
            ),
            (
                "$9$3ayas1CG7gPWd.$FF2Y5kel8EIjGlJTjw0kBu3GLtk2RIWIaO2Uiq3Q8vs",
                "6foo6",
            ),
            (
                "$9$02NxZc3g6VBz4U$OC9gBCsRyES802CP/A5DwFx1uAQt/rs2FQh7D6t9h/E",
                "7foo7",
            ),
            (
                "$9$mK1ofpcD0RU9ek$YMRifOqC95igMZnew84jvZfpjg9Ux68ez3L2Z0av8g2",
                "8foo8",
            ),
            (
                "$9$gSxJfegHuj0F6E$ENXdRsApTMICSTVFZcTWlNpYqcmWSEwcqsklANu6vb2",
                "9foo9",
            ),
            # C9300, IOS-XE 16.09.05
            (
                "$9$PXg4rSaFnoegO1$/6Rz3Wj5QFPep7vO1ZucuQX5m34V4riND55WJabgpPI",
                "0foo0",
            ),
            (
                "$9$WtRdvMXgJ.ucTH$aLBq6tLlZ5wQkaENKWNRVjj8lBcegs1NnMuT0ePzckE",
                "1foo1",
            ),
            (
                "$9$0pjeJLxng3RwIn$pVvqIza/0yx7OxwtPuxPnVo7zynzIF6KIfUqwSWLJks",
                "2foo2",
            ),
            (
                "$9$3Y/3AuJhoMgzW1$2cGvk9ttc0KDz4IpNwUBVLyl.SkXSOMirLvEnRRE0Qg",
                "3foo3",
            ),
            (
                "$9$Pm74lahwSPCK41$paN5FRwrc5IvjIDvqpfFoW8xeVmlBLi8OgAQWUX/6N2",
                "4foo4",
            ),
            (
                "$9$L83ULmQYYBEhbn$X0E3XEAlNJSuj1ONaVRy.45vrcR9fPudODlJWd3XSBo",
                "5foo5",
            ),
            (
                "$9$kdO8GOTC5on42X$Hvkr53ivGpUPHcoFOBXzB9I/E0ccFeOoBzdqjXBiBSI",
                "6foo6",
            ),
            (
                "$9$dSnrddpzAU9uhH$cMALHjMkiDsBztHRSPjZC0S0WyNYnC5nRHGQHtCVDGg",
                "7foo7",
            ),
            (
                "$9$dynV4r5F4XwFdH$yuN0E1i14DhOb04xilIp1lJ.N/IKJakDylS/EvMTc6w",
                "8foo8",
            ),
            (
                "$9$dddcyrMXj2f4tX$5wDEL2q.fmZRUP47ZlB9NpvMEgWwjtxMdYnhUH7kOmk",
                "9foo9",
            ),
        ]
        self.hash_password_seed_tuples = [
            (
                "$9$5A/0NQGf6wMdjN$4toTqNCPcBmsBgUsOxNy6uPLITQ4zKFfcf5sJuy3uMo",
                "asdfgsasdf",
                "asdfasdfasdfas",
            )
        ]



"""
This module provides hash functions for Cisco IOS and IOS-XE.

Currently only type 5 (md5-based) and type 9 (scrypt-based) are supported.
"""
from .type_5 import Type5
from .type_9 import Type9

__pdoc__ = {}
__pdoc__["type_5"] = False
__pdoc__["type_9"] = False

__all__ = ["Type5", "Type9"]



import scrypt
import base64

from network_hash_gen.base import BaseHash


class Type9(BaseHash):
    salt_length = 14
    salt_chars = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    _std_b64chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    _b64table = str.maketrans(_std_b64chars, salt_chars)

    @classmethod
    def hash_salted(cls, password: str, salt: str) -> str:
        """
        Calculates a Cisco IOS/IOS-XE Type 9 hash with the given password and salt.

        This function assumes that the given salt is valid for that hash.
        Using an invalid salt leads to undefined behaviour.
        Prefer the `type_9_hash` and `type_9_hash_seeded` functions over this one
        if possible.
        """

        hash = scrypt.hash(password, salt, 16384, 1, 1, 32)

        # Convert the hash from Standard Base64 to Cisco Base64
        hash = base64.b64encode(hash).decode().translate(cls._b64table)[:-1]

        return f"$9${salt}${hash}"



from setuptools import setup, find_packages
from network_hash_gen import __version__

with open("README.md") as f:
    readme = f.read()

setup(
    name="network_hash_gen",
    version=__version__,
    packages=find_packages(),
    url="https://github.com/991jo/network_hash_gen",
    author="Johannes Erwerle",
    author_email="jo+network_hash_gen@swagspace.org",
    description="A library to generate hashes for network devices.",
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires=["passlib == 1.7.4", "scrypt == 0.8.20"],
    extras_require={
        "dev": ["black", "coverage"],
        "docs": ["pdoc3"],
        "publish": ["twine"],
    },
)



from unittest import TestCase


class BaseHashTest:
    class HashTest(TestCase):
        """
        This class runs several tests for a group of function belonging to a hash
        scheme.

        This group is expected to be a class having the following classmethods:
        - `hash`
        - `hash_salted`
        - `hash_seeded`
        - `_generate_salt`

        This class has the following instance variables, which need to be set:

        - `group`: the class representing the hash scheme
        - `hash_password_pairs: List(Tuple(str, str))` A list of hash and password pairs.
          They are used to test whether the salted hash version returns the correct
          hashes. The hashes here are usually taken from device configs.
        - `hash_password_seed_tuples: List(Tuple(str, str, str))` A list of hashes,
          passwords and seeds. This is used to ensure that the seeded version of the
          hash returns the same values in the future.
        """

        def test_hash_pairs(self):

            assert getattr(self, "group") is not None
            assert getattr(self, "hash_password_pairs") is not None

            assert len(self.hash_password_pairs) > 0

            for hash, plaintext in self.hash_password_pairs:
                salt = hash.split("$")[2]
                self.assertEqual(hash, self.group.hash_salted(plaintext, salt))

        def test_hash_password_seed_tuples(self):

            assert getattr(self, "group") is not None
            assert getattr(self, "hash_password_seed_tuples") is not None

            assert len(self.hash_password_seed_tuples) > 0

            for hash, password, seed in self.hash_password_seed_tuples:
                self.assertEqual(hash, self.group.hash_seeded(password, seed))



from typing import Optional
import random


def _generate_salt(chars: str, length: int, seed: Optional[str] = None) -> str:
    """
    Generates a salt with the specified `length`, chosen randomly from the
    charaters in `chars`

    If `seed` is specified the value is used to initialize the random number
    generator.
    """

    # initialize a seperate RNG to no seed the global instance used by the
    # functions called directly on the random module.
    rng = random.Random()

    if seed is not None:
        rng.seed(seed)

    output = ""
    for _ in range(length):
        output += rng.choice(chars)

    return output
